/* src/test/modules/test_extensions/test_ext_cyclic1--1.0.sql */
-- complain if script is sourced in psql, rather than via CREATE EXTENSION
\echo Use "CREATE EXTENSION test_ext_cyclic1" to load this file. \quit
